@@ -19,6 +19,7 @@ from osk import OnScreenKeyboard
 from floating import ALIGNMENTS
 from util import utils, trackers, settings
 from util.eventHandler import EventHandler
+from util.utils import DEBUG
 
 class Stage(Gtk.Window):
     """
@@ -59,9 +60,10 @@ class Stage(Gtk.Window):
         self.destroying = False
 
         self.manager = manager
-        status.screen = CScreensaver.Screen.new(status.Debug)
         self.away_message = away_message
 
+        self.activate_callback = None
+
         self.monitors = []
         self.last_focus_monitor = -1
         self.overlay = None
@@ -72,8 +74,6 @@ class Stage(Gtk.Window):
         self.info_panel = None
         self.osk = None
 
-        self.stage_refresh_id = 0
-
         self.floaters = []
 
         self.event_handler = EventHandler(manager)
@@ -93,7 +93,7 @@ class Stage(Gtk.Window):
                         Gdk.EventMask.FOCUS_CHANGE_MASK)
 
         c = Gdk.RGBA(0, 0, 0, 0)
-        self.override_background_color (Gtk.StateFlags.NORMAL, c);
+        self.override_background_color(Gtk.StateFlags.NORMAL, c)
 
         self.update_geometry()
 
@@ -158,8 +158,7 @@ class Stage(Gtk.Window):
         Wallpapers are secondary.
         """
 
-        if status.Debug:
-            print("Stage: Received screen size-changed signal, refreshing stage")
+        DEBUG("Stage: Received screen size-changed signal, refreshing stage")
 
         self.emit("needs-refresh")
 
@@ -169,25 +168,13 @@ class Stage(Gtk.Window):
         as on_screen_size_changed), and follow up at idle with actual monitor view
         refreshes (wallpapers.)
         """
-        if status.Debug:
-            print("Stage: Received screen monitors-changed signal, refreshing stage")
+        DEBUG("Stage: Received screen monitors-changed signal, refreshing stage")
 
         self.emit("needs-refresh")
 
     def on_composited_changed(self, screen, data=None):
         if self.get_realized():
-            if status.Debug:
-                print("Stage: Received screen composited-changed signal, refreshing stage")
-
-            user_time = self.get_display().get_user_time()
-
-            self.hide()
-            self.unrealize()
-
-            self.realize()
-
-            self.get_window().set_user_time(user_time)
-            self.show()
+            DEBUG("Stage: Received screen composited-changed signal, refreshing stage")
 
             self.emit("needs-refresh")
 
@@ -196,35 +183,6 @@ class Stage(Gtk.Window):
 
         return False
 
-    def queue_refresh_stage(self):
-        """
-        Queues a complete refresh of the stage, resizing the screen if necessary,
-        reconstructing the individual monitor objects, etc...
-        """
-        if self.stage_refresh_id > 0:
-            GObject.source_remove(self.stage_refresh_id)
-            self.stage_refresh_id = 0
-
-        self.stage_refresh_id = GLib.idle_add(self._update_full_stage_on_idle, priority=GLib.PRIORITY_DEFAULT)
-
-    def _update_full_stage_on_idle(self, data=None):
-        self.stage_refresh_id = 0
-        self._refresh()
-
-        return False
-
-    def _refresh(self):
-        Gdk.flush()
-        if status.Debug:
-            print("Stage: refreshing")
-
-        self.update_geometry()
-        self.move_onscreen()
-        self.update_monitors()
-        self.overlay.queue_resize()
-
-        self.manager.stage_refreshed()
-
     def activate(self, callback):
         """
         This is the primary way of making the Stage visible.
@@ -234,7 +192,10 @@ class Stage(Gtk.Window):
         self.move_onscreen()
         self.show()
 
-        callback()
+        if self.get_realized():
+            callback()
+        else:
+            self.activate_callback = callback
 
     def deactivate(self, callback):
         """
@@ -254,14 +215,19 @@ class Stage(Gtk.Window):
         window = self.get_window()
         utils.override_user_time(window)
 
+        CScreensaver.Screen.set_net_wm_name(window, "cinnamon-screensaver-window")
+
         self.setup_children()
 
-        self.gdk_filter.start()
+        self.gdk_filter.start(singletons.MuffinClient.get_using_fractional_scaling(), status.Debug)
 
         trackers.con_tracker_get().disconnect(self.overlay,
                                               "realize",
                                               self.on_realized)
 
+        if self.activate_callback is not None:
+            self.activate_callback()
+
     def move_onscreen(self):
         w = self.get_window()
 
@@ -339,39 +305,39 @@ class Stage(Gtk.Window):
             print(e)
 
         try:
-            if self.unlock_dialog != None:
+            if self.unlock_dialog is not None:
                 self.unlock_dialog.destroy()
         except Exception as e:
             print(e)
 
         try:
-            if self.clock_widget != None:
+            if self.clock_widget is not None:
                 self.clock_widget.stop_positioning()
                 self.clock_widget.destroy()
         except Exception as e:
             print(e)
 
         try:
-            if self.albumart_widget != None:
+            if self.albumart_widget is not None:
                 self.albumart_widget.stop_positioning()
                 self.albumart_widget.destroy()
         except Exception as e:
             print(e)
 
         try:
-            if self.info_panel != None:
+            if self.info_panel is not None:
                 self.info_panel.destroy()
         except Exception as e:
             print(e)
 
         try:
-            if self.info_panel != None:
+            if self.info_panel is not None:
                 self.audio_panel.destroy()
         except Exception as e:
             print(e)
 
         try:
-            if self.osk != None:
+            if self.osk is not None:
                 self.osk.destroy()
         except Exception as e:
             print(e)
@@ -446,7 +412,6 @@ class Stage(Gtk.Window):
                                               self.position_overlay_child)
 
         self.destroy()
-        status.screen = None
 
     def setup_monitors(self):
         """
@@ -500,10 +465,9 @@ class Stage(Gtk.Window):
         Callback for UPower changes, this will make our MonitorViews update
         themselves according to user setting and power state.
         """
-        if status.Debug:
-            print("stage: Power state changed, updating info panel")
+        DEBUG("stage: Power state changed, updating info panel")
 
-        if self.info_panel != None:
+        if self.info_panel is not None:
             self.info_panel.update_visibility()
 
     def setup_clock(self):
@@ -645,9 +609,9 @@ class Stage(Gtk.Window):
         widget, depending on the outcome.
         """
         if success:
-            if self.clock_widget != None:
+            if self.clock_widget is not None:
                 self.clock_widget.hide()
-            if self.albumart_widget != None:
+            if self.albumart_widget is not None:
                 self.albumart_widget.hide()
             self.unlock_dialog.hide()
             self.manager.unlock()
@@ -661,7 +625,7 @@ class Stage(Gtk.Window):
         """
         Passes along an away-message to the clock.
         """
-        if self.clock_widget != None:
+        if self.clock_widget is not None:
             self.clock_widget.set_message(msg)
 
     def initialize_pam(self):
@@ -680,9 +644,9 @@ class Stage(Gtk.Window):
 
         utils.clear_clipboards(self.unlock_dialog)
 
-        if self.clock_widget != None:
+        if self.clock_widget is not None:
             self.clock_widget.stop_positioning()
-        if self.albumart_widget != None:
+        if self.albumart_widget is not None:
             self.albumart_widget.stop_positioning()
 
         status.Awake = True
@@ -690,18 +654,18 @@ class Stage(Gtk.Window):
         if self.info_panel:
             self.info_panel.refresh_power_state()
 
-        if self.clock_widget != None:
+        if self.clock_widget is not None:
             self.clock_widget.show()
-        if self.albumart_widget != None:
+        if self.albumart_widget is not None:
             self.albumart_widget.show()
 
         self.unlock_dialog.show()
 
-        if self.audio_panel != None:
+        if self.audio_panel is not None:
             self.audio_panel.show_panel()
-        if self.info_panel != None:
+        if self.info_panel is not None:
             self.info_panel.refresh_power_state()
-        if self.osk != None:
+        if self.osk is not None:
             self.osk.show()
 
     def cancel_unlocking(self):
@@ -711,24 +675,24 @@ class Stage(Gtk.Window):
         self.set_timeout_active(None, False)
         utils.clear_clipboards(self.unlock_dialog)
 
-        if self.unlock_dialog != None:
+        if self.unlock_dialog is not None:
             self.unlock_dialog.hide()
-        if self.clock_widget != None:
+        if self.clock_widget is not None:
             self.clock_widget.hide()
-        if self.albumart_widget != None:
+        if self.albumart_widget is not None:
             self.albumart_widget.hide()
-        if self.audio_panel != None:
+        if self.audio_panel is not None:
             self.audio_panel.hide()
-        if self.info_panel != None:
+        if self.info_panel is not None:
             self.info_panel.hide()
-        if self.osk != None:
+        if self.osk is not None:
             self.osk.hide()
 
         status.Awake = False
 
         self.update_monitor_views()
 
-        if self.info_panel != None:
+        if self.info_panel is not None:
             self.info_panel.refresh_power_state()
 
     def update_monitor_views(self):
@@ -738,9 +702,9 @@ class Stage(Gtk.Window):
         """
 
         if not status.Awake:
-            if self.clock_widget != None and settings.get_show_clock():
+            if self.clock_widget is not None and settings.get_show_clock():
                 self.clock_widget.start_positioning()
-            if self.albumart_widget != None and settings.get_show_albumart():
+            if self.albumart_widget is not None and settings.get_show_albumart():
                 self.albumart_widget.start_positioning()
 
         for monitor in self.monitors:
@@ -784,8 +748,11 @@ class Stage(Gtk.Window):
         else:
             self.rect = status.screen.get_screen_geometry()
 
-        if status.Debug:
-            print("Stage.update_geometry - new backdrop position: %d, %d  new size: %d x %d" % (self.rect.x, self.rect.y, self.rect.width, self.rect.height))
+            scale = status.screen.get_global_scale()
+            self.rect.width *= scale
+            self.rect.height *= scale
+
+        DEBUG("Stage.update_geometry - new backdrop position: %d, %d  new size: %d x %d" % (self.rect.x, self.rect.y, self.rect.width, self.rect.height))
 
         hints = Gdk.Geometry()
         hints.min_width = self.rect.width
@@ -912,7 +879,7 @@ class Stage(Gtk.Window):
                 the unlock dialog, reduce our available region width to accomodate it, reducing
                 the allocation for the floating widgets as required.
                 """
-                if (unlock_nw > region_w):
+                if unlock_nw > region_w:
                     region_w = (monitor_rect.width - unlock_nw) / 2
 
                 region_h = monitor_rect.height
